<?php
/**
 * Handling for files
 * @author Andreas Plank <andreas.plank@web.de>
 * @version 2009-04-05 15:53:30
 * @license Noncommercial-Share Alike 3.0 Unported
 * @category program tools
 * @package chirokey_adults
 * @subpackage program
 */

/**
 * Classes used for handling files (writing/reading)
 *
 * @author Andreas Plank <andreas.plank@web.de>
 * @version 2009-04-05 15:53:30
 * @license Noncommercial-Share Alike 3.0 Unported
 * @category program tools
 * @package chirokey_adults
 * @subpackage program
 */
class fileWriteDelete
{
  /**
  * Directory to read
  *
  * @var string
  * @access public
  */
  public $directory=".";
  /**
  * Error messages
  *
  * Messages are generated by {@link fileWriteDelete::errors()}
  * @var string
  * @access public
  */
  public $error_message="" ;
  /**
  * Error number
  *
  * Messages are generated by {@link fileWriteDelete::errors()}
  * @var array
  * @access public
  */
  public $error_no=0;
  public $content;
  private $strSomething='default';

  /**
    * File's extension/data
    * @var array
    * @access public
    */
  public $file_extension;
  /**
    *  Uploded file
    *
    * It comes usually from PHP directory like /tmp/php5PBI4u
    * @var string
    * @access private
    */
  public $strFileInput;
  public $file_binary;
  public $strReferenceName;
  /**
  * Destination to write the file
  *
  * @var string
  * @access public
  */
  public $strFilename;
  public $source;
  public $directory_name;

   /**
     * Test directory if it can be opend or is writable
     */
  public function __construct($directory) {
    if(!opendir($directory))
    { self::writeDirectory($directory);
      self::errors(1,$directory);// exit skript here
    }
    $this->handle_0=opendir($directory);
    if(chdir($directory)==false)//chdir supply FALSE or TRUE
    {closedir($this->handle_0);
      self::errors(6,$directory);// exit skript here
    }
    /* TEST ?is writable */
    if(!is_writeable('.'))
    {closedir($this->handle_0);
      self::errors(0,$directory);// exit skript here
    }
  }// end function __construct

   /**
     * Close directory reading
     */
  public function __destruct() {
    if(opendir($directory))
    {
    closedir($this->handle_0);
    }
  }

   /**
     * Some error messages
     *
     * Errors appering while reading writing files. Uses CSS class 'error'
     * <code><div class="error">Some error message...</div></code>
     */

  function errors($which_error,$error_message){
    /**
    * Read page structure
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'pages.php');
    /**
    * Read config
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
    $error_no[0]  = "\n<div  class='error'>Error: directory <span class='emphasize'>'".$error_message."'</span> isn't writeable for the server.</div>";
    $error_no[1]  = "\n<div  class='error'>Error while open directory: <span class='emphasize'>'".$error_message."'</span> couldn't be opend. It does not exist.<br>A new one was created named: '".$error_message."'</div>";
    $error_no[2]  = "\n<div  class='error'>Error while writing in directory: couldn't write into file <span class='emphasize'>'".$error_message."'.</span></div>";
    $error_no[3]  = "\n<div  class='error'>Error with image properties:  height/width from file <span class='emphasize'>'".$error_message."' is more than ".$config['img_max_size']."&nbsp;px.</span></div>";
    $error_no[4]  = "\n<div  class='error'>Error: wrong filetype from file <span class='emphasize'>'".$error_message."'.</span></div>";
    $error_no[5]  = "\n<div  class='error'>Error: Sorry maximum filesize is ".$config['img_max_filesize']."&nbsp;kB. File <span class='emphasize'>'".$error_message."'.</span></div>";
    $error_no[6]  = "\n<div  class='error'>Error: couldn't change to directory <span class='emphasize'>'".$error_message."'.</span></div>";
    $error_no[7]  = "\n<div  class='error'>Error: There is no Author/Reference given for <span class='emphasize'>'".$error_message."'.</span></div>";
    $error_no[8]  = "\n<div  class='error'>Error: Can't initialize new GD image stream for reference picture <span class='emphasize'>'".$error_message."'.</span></div>";
    $error_no[9]  = "\n<div  class='error'>Error: Can't delete file <span class='emphasize'>'".$error_message."'.</span></div>";
    $error_no[10] = "\n<div  class='error'>Error: Can't write file <span class='emphasize'>'".$error_message."'</span>: it already exists in database. Select overwrite 'yes'.<br> <a href='javascript:history.back();void(0);'>back</a></div>\n<div class='notice'>\n  <img src='".$config['BaseUrl'].$config['dirSpec'].$error_message."' style='vertical-align:text-top;'>\n</div>";
    //output error
    echo $error_no[$which_error].$page['foot'];
    clearstatcache();// release memory
    exit;// exit skript here
  }

/**
 * Writes a file_r.jpeg, file_r.png or file_r.gif
 *
 * Writes an image file with a reference name where '_r' means is referenced.
 * It is used e.g. for uploads.
 * @param string $strFilename
 * @param string $strReferenceName
 * @param image $content from PHP variable $_FILES['uploadfile']['tmp_name']
 */

  function write($strFilename, $strReferenceName, $content)
  {//$content : imagename
    /* add file extension */
    $file_extension=getimagesize($content);
    $file_extension['mime']=split("/",$file_extension['mime']);
    $strFilename=$strFilename."_r.".$file_extension['mime'][1];// supplies file_r.jpeg file_r.png '_r'->means referenced
    /* TEST possible to open file */
    $fileHandle=fopen($strFilename,'wb');
    if(!$fileHandle)
    {
      self::errors(1,$strFilename);// exit skript here
    }
    /* TEST is writeable */
    if(!fwrite($fileHandle,$content))
    {fclose($fileHandle);
      self::errors(2,$strFilename);// exit skript here
    }

    $this->strFilename=$strFilename;
    /* write Reference */
    self::writeReference($content, $strReferenceName, $strFilename);
    /* OK finally close directory */
      fclose($fileHandle);
    //exit;// exit all following skripts here
  }//end function file_write

/**
 * Test whether it is an image
 *
 * This function tests also some settings from the configuration file. If it fails script stops with
 * <code>exit;</code>
 * @param binary $file_binary
 */
  function validateImage($file_binary)
  {
    /**
    * Read config
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
    /*$config['img_max_filesize'] = "100"; //  kB
    $config['img_max_size'] = "600";// max width/height in px
    $config['img_types'][0]="image/png";
    $config['img_types'][1]="image/jpeg";*/
    $file_getimagesize=getimagesize($file_binary);
    /*
     [0] => 440
     [1] => 278
     [2] => 3 type shorten
     [3] => width="440" height="278"
     [bits] => 8
     [channels] => 3 only if jpeg
     [mime] => image/jpeg
     */
    // print_r($_FILES);// Test uploaded files
    //TEST mime type
    if(!in_array($file_getimagesize['mime'],$config['img_types'])  )
    {
      self::errors(4,$file_binary);// exit skript here
    }
    //TEST img_max_size see in config.php $config['img_max_size']
    if($file_getimagesize[0] + $file_getimagesize[1] > 2*$config['img_max_size'])
    {
      self::errors(3,$file_binary." (w:".$file_getimagesize[0]." h:".$file_getimagesize[1].")");// exit skript here
    }
    //TEST img_max_filesize from config.php
    if(filesize($file_binary)/1024 > $config['img_max_filesize'])
    {
      self::errors(5,$file_binary." is ".round(filesize($file_binary)/1024,2)."&nbsp;kB");// exit skript here
    }
  }//end function file_validate

/**
 * Writes a reference into the image file.
 *
 * An image file is referenced when the reference name is included into the image.
 * So to say as sub caption.
 * @param string $strFileInput from PHP directory /tmp/php5PBI4u
 * @param string $strReferenceName name of reference (default short authors' names)
 * @param string $strFilename whole name of file without path, e.g.: Prcl_chor_hes_Rie2001_r.gif
 * @return image
 */
  function writeReference($strFileInput,$strReferenceName,$strFilename)
  {# needs config
    /**
    * Read config
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
    $imagesize=getimagesize($strFileInput);//print_r($imagesize);exit;//is array
    $im_origwidth=$imagesize[0];
    $im_orighight=$imagesize[1];
    /*
    [0] => 440 width
    [1] => 278 height
    [2] => 3 type shorten
    [mime] => image/jpeg
    */
    $strReferenceName=html_entity_decode($strReferenceName,ENT_QUOTES);
    if(empty($strReferenceName))
    {/*if empty reference text -> blinking Text*/
      self::errors(7,$strFileInput);/* stop execution */
    }
    else
    {/* create a referenced picture */
      //mesure text: array<-imagettfbbox ( int size, int angle, string fontfile, string text )
      $text_mesure_orig=imagettfbbox($config['fontsize'],0,$config['fontpath'],$strReferenceName);
      /* ______________________
        |  6-oliX     4-oreX   |
        7-oliY            5-oreY
        |                      |
        1-uliY            3-ureY
        |__0-uliX_____2-ureX __|
        Pinder and Reiss 1983 -> [2]=>94 [4]=>94 ->width [1] y-Anfang-> -1 [7] y-Ende-> neg.
        */
      //print_r($text_mesure);exit;
      $im_ratio=$im_orighight/$im_origwidth;// portrait > 1     landscape < 1
      if($im_origwidth < $text_mesure_orig[2])
      {/* too long Text */
        if($im_ratio > 1)//portrait
        {/* img with rotated text */
          $wrap_width = @round($im_orighight*strlen($strReferenceName)/$text_mesure_orig[2],0);
          $strReferenceName=@wordwrap($strReferenceName,$wrap_width);/* wrap lines */
          $text_mesure=imagettfbbox($config['fontsize'], 0, $config['fontpath'], $strReferenceName);
          //new reference picture
          $im_newwidth=$im_origwidth-$text_mesure[7]+$text_mesure[1];
            //echo "\$im_origwidth < \$text_mesure_orig<br>portrait > 1:<br>";print_r($text_mesure);
          $im_newheight=$im_orighight;
          $ref_im = @ImageCreatetruecolor($im_newwidth, $im_newheight); $angle=90;
          if(!$ref_im) self::errors(8,file);
        }// end portrait
        else//landscape
        {/* img with normal not rotated text */
          $wrap_width = @round($im_origwidth*strlen($strReferenceName)/$text_mesure_orig[2],0);
          $strReferenceName=@wordwrap($strReferenceName,$wrap_width);/* wrap lines */
          $text_mesure=imagettfbbox($config['fontsize'], 0, $config['fontpath'], $strReferenceName);
          //new reference picture
            $im_newwidth=$im_origwidth;
            $im_newheight=$im_orighight-$text_mesure[7]+$text_mesure[1];
          $ref_im = @ImageCreatetruecolor($im_newwidth, $im_newheight); $angle=0;
          if(!$ref_im) self::errors(8,file);
            //echo "\$im_origwidth < \$text_mesure_orig<br>landscape < 1:<br>";print_r($text_mesure);
        }//end landscape
      }// end too long Text width
      else
      {/* text shorter than im_origwidth */
          $im_newwidth=$im_origwidth;
          $im_newheight=$im_orighight-$text_mesure_orig[7]+$text_mesure_orig[1];
        $ref_im = @ImageCreatetruecolor($im_newwidth, $im_newheight);$angle=0;
        if(!$ref_im) self::errors(8,file);
          //echo "\$im_origwidth > \$text_mesure_orig<br>portrait/landscape < 1:<br>";print_r($text_mesure);
      }/* end text shorter than im_origwidth */

      /* background_color in $ref_im */
      $background_color = ImageColorAllocate ($ref_im, 255, 255, 255);
      imagefill($ref_im,0,0,$background_color);//fill ref_im

      /* which picture type */
      switch ($imagesize['mime']) {
        case 'image/png':
          $im_orig = @imageCreateFromPng($strFileInput); break;
        case 'image/jpeg':
          $im_orig = @imagecreatefromjpeg($strFileInput); break;
        case 'image/gif':
          $im_orig = @imagecreatefromgif($strFileInput); break;
      }

      /* new ref-picture */
      //imagecopyresized ( resource dst_im, resource src_im, dstX, dstY, srcX, srcY, dstW, dstH, srcW, srcH)
      imagecopyresized($ref_im, $im_orig,0,0,0,0, $imagesize[0], $imagesize[1], $im_origwidth, $im_orighight);

      /* echo "text_mesure: [1]".$text_mesure[1]." [7]".$text_mesure[7]."<br>";
      echo "im_newwidth: ".$im_newwidth." im_newheight: ".$im_newheight."<br>";
      echo "imagesize: [0]".$imagesize[0]." imagesize: [1]".$imagesize[1]."<br>";
      print_r($text_mesure); */

      $text_color = ImageColorAllocate ($ref_im, 0, 0, 0);

      if($im_ratio > 1)
      {//portrait
        if($im_origwidth > $text_mesure_orig[2])
        {/* reference_name fit into picture */
          imagettftext($ref_im,$config['fontsize'],$angle,1,$im_newheight-1, $text_color, $config['fontpath'], $strReferenceName);
        }
        else
        {/* reference_name is longer than picture */
          imagettftext($ref_im,$config['fontsize'],$angle, $im_origwidth-$text_mesure_orig[7]-1, $im_orighight, $text_color, $config['fontpath'],$strReferenceName);
        }
      }
      else
      {//landscape
        imagettftext($ref_im,$config['fontsize'], $angle, 1, $im_orighight-$text_mesure_orig[7]-1, $text_color, $config['fontpath'], $strReferenceName);
      }

      // write finally file to destination
      switch ($imagesize['mime']) {
        case 'image/png':
          imagePng($ref_im, $strFilename, 9);
        case 'image/gif':
          imageGif($ref_im, $strFilename); break;
        case 'image/jpeg':
          imagejpeg($ref_im, $strFilename, 100); break;
      }
      // release memory, delete picture
      imagedestroy($ref_im);
    }//end $strReferenceName not empty
  }// end function file_write_reference

  /**
  * Deletes an image file.
  *
  * @param string $source ?path to the deleted file
  */
  function deleteFile($source){
  if(unlink(basename($source))==false) self::errors(9,$source);
  $this->strFilename=basename($source);
  }// end function file_delete

  /**
  * Create a directory with full rights (mode=0777)
  *
  * @param string $directory_name
  * @param string $mode default 0777
  */
  function writeDirectory($directory_name,$mode=0777){
    mkdir($directory_name,$mode);
  }//end directory_write
}// end class fileWriteDelete

 ?>