<?php
/**
 * Communicate with the database
 *
 *
 * @author Andreas Plank <andreas.plank@web.de>
 * @copyright Creative Commons Noncommercial 3.0
 * @package chirokey_adults
 * @subpackage database
 * @version 2009-02-24 21:24:33
 */

/**
 * Classe for communicate with the database
 *
 * Gets data from database with utf8 connenction. Class is modified from the german book: Michael Kofler and Bernd Öggl: PHP 5 & MySQL
 * 5 - Grundlagen, Programmiertechniken, Beispiele. Addison-Wesley. 2006
 * @author Andreas Plank <andreas.plank@web.de>
 * @copyright Creative Commons Noncommercial 3.0
 * @package chirokey_adults
 * @subpackage database
 * @version 2009-02-24 21:24:33
 */

class db_mysql {
  protected $mysqli;
  protected $showerror = TRUE;   // set FALSE if you don't want to see error messages
  public    $showsql   = FALSE;  // set TRUE if you want to see all SQL queries for debugging purposes
  protected $sqlcounter = 0;     // counter for SQL commands
  protected $rowcounter = 0;     // counter for returned SELECT rows
  protected $dbtime     = 0;     // counter for time needed to execute queries
  protected $starttime;
  public    $doReturn = FALSE; // do return for print_r();
/**
 * Prepare MySQL database connenction
 *
 * Reads configuration settings and establishes an utf8 connenction.
 * Provides time summarization. Works with PHP class mysqli.
 */

  function __construct() {
    /**
    * Extract current server path relatively 1-level up: {1}
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
    //require_once($_SERVER['DOCUMENT_ROOT'].'/sign_language/config.inc.php');
    $this->mysqli = @new mysqli($config['dbserver'],$config['dbuser'],$config['dbpassword'],$config['database']);
    //utf8 support
    $this->mysqli->query("SET @@session.character_set_client  = 'utf8'");
    $this->mysqli->query("SET @@session.character_set_results = 'utf8'");
    $this->mysqli->query("SET @@character_set_connection      = 'utf8'");
    $this->mysqli->query("SET @startcount      = 0"); // use as rowcounter etc.
    // testen, ob Verbindung OK
    if(mysqli_connect_errno()) {
      #html_start('MySQL connection error','error');
      $this->printerror("Sorry, no connection to MySQL! (" . mysqli_connect_error() . ")");
      #html_end();
      $this->mysqli = FALSE;
      exit();
    }
    $this->starttime = $this->microtime_float();
  }

/**
 * close database connenction
 */
  function __destruct() {
    $this->close();
   }

/**
 * explicit close
 * @return FALSE
 */
  function close() {
    if($this->mysqli)
      $this->mysqli->close();
      $this->mysqli = FALSE;
  }

/**
 * Returns $mysqli
 */
  function getMysqli() {
    return $this->mysqli;
  }

/**
 * Execute SELECT query
 *
 * This returns an class object. You can get results with <code>$objMyDatabase->columnName</code>
 * @return objArray|FALSE
 * @param $sql an SQL string
 */
  function queryObjectArray($sql) {
    $this->sqlcounter++;
    $this->printsql($sql);
    $time1  = $this->microtime_float();
    $result = $this->mysqli->query($sql);
    $time2  = $this->microtime_float();
    $this->dbtime += ($time2 - $time1);
    if($result) {
      if($result->num_rows) {
        while($row = $result->fetch_object())
          $result_array[] = $row;
        $this->rowcounter += sizeof($result_array);
        return $result_array; }
      else
        return FALSE;
    } else {
      $this->printerror($this->mysqli->error);
      $this->showsql = TRUE;
      $this->printsql($sql);
      return FALSE;
    }
  }

/**
 * Execute SELECT query
 * @return array|FALSE
 * @param $sql an SQL string
 */
  function queryArray($sql) {
    $this->sqlcounter++;
    $this->printsql($sql);
    $time1  = $this->microtime_float();
    $result = $this->mysqli->query($sql);
    $time2  = $this->microtime_float();
    $this->dbtime += ($time2 - $time1);
    if($result) {
      if($result->num_rows) {
        while($row = $result->fetch_array())
          $result_array[] = $row;
        $this->rowcounter += sizeof($result_array);
        return $result_array; }
      else
        return FALSE;
    } else {
      $this->printerror($this->mysqli->error);
      $this->showsql = TRUE;
      $this->printsql($sql);
      return FALSE;
    }
  }


/**
 * Return a single item
 *
 * Only a single item, e.g.:<code>SELECT COUNT(*) FROM table;</code>
 * Attention: this method returns -1 for no data and false for an error.
 * @return string|-1|FALSE
 * @param $sql an SQL string
 */

  function querySingleItem($sql) {
    $this->sqlcounter++;
    $this->printsql($sql);
    $time1  = $this->microtime_float();
    $result = $this->mysqli->query($sql);
    $time2  = $this->microtime_float();
    $this->dbtime += ($time2 - $time1);
    if($result) {
      if ($row=$result->fetch_array()) {
        $result->close();
        $this->rowcounter++;
        return $row[0];
      } else {
        // query returned no data
        return -1;
      }
    } else {
      $this->printerror($this->mysqli->error);
      $this->showsql = TRUE;
      $this->printsql($sql);
      return FALSE;
    }
  }


/**
 * Execute a SQL command no *results* returned
 * @return boolean
 * @param $sql an SQL string
 */
  function execute($sql) {
    $this->sqlcounter++;
    $this->printsql($sql);
    $time1  = $this->microtime_float();
    $result = $this->mysqli->real_query($sql);
    $time2  = $this->microtime_float();
    $this->dbtime += ($time2 - $time1);
    if($result)
      return TRUE;
    else {
      $this->printerror($this->mysqli->error);
      $this->showsql = TRUE;
      $this->printsql($sql);
      return FALSE;
    }
  }


/**
 * Get inserted ID after INSERT
 * @return void
 */
  function insertId() {
    return $this->mysqli->insert_id;
  }

/**
 * Inserts a \ before ', " etc.
 * @return mixed
 */
  function escape($txt) {
    return trim($this->mysqli->escape_string($txt));
  }

/**
 * Escapes strings for save
 *
 * @return string
 * @param string $txt text that will be escaped: 'NULL' if empty or 'quoted \'Strings\''
 */

  function sql_string($txt) {
    if(!$txt || trim($txt)=="")
      return 'NULL';
    else
      return "'" . $this->escape(trim($txt)) . "'";
  }

/**
 * Returns errors
 * @return see mysqli
 */
  function error() {
    return $this->mysqli->error;
  }

/**
 * Print SQL as output
 *
 * Output is given as CSS class 'sql_output':
 * <code><p class="sql_output"><span>...</span></p></code>
 * A simple highlighting is done.
 * @return HTML
 * @param $sql an SQL string
 */
  private function printsql($sql) {
    if($this->showsql){
      $ln = 1;
      $sql = preg_replace(
        array(
          "#^(.+)$#me",
          "#\b(ALTER|ANALYSE|ANALYZE|AND|ASC|AS|DELETE|DESCRIBE|DESC|DISTINCT|DROP|DUPLICATE|EXPLAIN|FROM|HOUR|IGNORE|INDEX|INSERT|INTERVAL|INTO|IS|KEY|LEFT +JOIN|LIMIT|RIGHT +JOIN|LIKE|NOT|NULL|ON|ORDER +BY|OR|PRIMARY|PROCEDURE|REFERENCES|REGEXP|SELECT|SET|SHOW|STRAIGHT_JOIN|TRUNCATE|UNIQUE|UPDATE|USE|USING|VALUES|WHERE)\b#",
          "#\b(CHAR_LENGTH|CONCAT|IFNULL|IF|IN|REPLACE|SUBSTRING|STRCMP)\b#",
          "#'([^']*)'#U",
          "#`([^`]+)`#",
          "#(\-\-.+)</code>$#m"
        ),
       array(
          "'    <code title=\"'.sprintf('%02s',\$ln++).'\" >\\1</code>'",
          "<span class=\"sqlkeyword\">\\1</span>",
          "<span class=\"sqlfunction\">\\1</span>",
          "<span class=\"sqlstring\">&#039;\\1&#039;</span>",
          "<span class=\"sqlfield\">&#096;\\1&#096;</span>",
          "<span class=\"comment\">\\1</span></code>"
        ),
        htmlentities($sql)
      );
      $sql=sprintf("\n  <p class=\"sql_output\">\n  <pre class='sourcecode'>%s  </pre>\n  </p>\n", $sql);
      echo ($sql);
    }
  }

/**
 * Prints SQL-ERROR
 * Output is given as CSS class 'sql_error':
 * <code><p class="sql_error"><span >...</span></p></code>
 * @return HTML
 */
  private function printerror($txt) {
    if($this->showerror){
      printf("\n  <p class=\"sql_error\"><span>%s</span></p>\n",
        nl2br(htmlentities($txt)));
    }
  }

/**
 * Print statistics
 *
 * Output is given as CSS class 'info':
 * <code><div class='info'>...</div></code>
 * @return HTML
 */
  function showStatistics($doReturn=false) {
    $totalTime = $this->microtime_float() - $this->starttime;
    $content.=sprintf("\n<div class='info'>  <p class=\"dbstatistic\">Some debugging informations<br><span color=\"#0000ff\">SQL commands: %d\n",      $this->sqlcounter);
    $content.=sprintf("<br >Sum of returned rows: %d\n",      $this->rowcounter);
    $content.=sprintf("<br >Sum of query time (MySQL): %fs\n", $this->dbtime);
    $content.=sprintf("<br >Processing time (PHP): %fs\n",     $totalTime - $this->dbtime);
    $content.=sprintf("<br >Total time since MyDB creation / last reset: %fs</span></p>\n</div>", $totalTime);
    if($doReturn){
      return print_r($content, $doReturn);
      }else{
      print_r($content);
    }
  }// end showStatistics()

/**
 * Set statistics to zero
 * @return HTML
 */
  function resetStatistics() {
    $this->sqlcounter = 0;
    $this->rowcounter = 0;
    $this->dbtime     = 0;
    $this->starttime = $this->microtime_float();
  }
/**
 * Timing function
 *
 * Execution with microtime()-example in PHP doc
 * @return string
 */
  private function microtime_float() {
    list($usec, $sec) = explode(" ", microtime());
    return ((float)$usec + (float)$sec);
  }

}// end class db_mysql
 ?>