<?php
/**
 * Some useful functions
 *
 * @author Andreas Plank <andreas.plank@web.de>
 * @version 2009-02-24 21:24:33
 * @license Noncommercial-Share Alike 3.0 Unported
 * @category program tools
 * @todo minor issue: document further detailed
 * @package chirokey_larvae
 * @subpackage program
 */

/**
 * Print a pretty table to save
 *
 * Used for references. Usage eg.:
 * <code>call_user_func('tabprint', $SQL-Result-Array, MySQL-Table-Definition, $addStructure)</code>
 * It tries to change fields according to MySQL table fields
 * @param array $arrTabFieldData values from MySQL data table
 * @param array $arrMysqlTableStructure MySQL table structure
 * @param array $addStructure additional HTML elements as array. Array key = data base field
 * e.g.<code>array(
 *     'name_author' => ' disabled class="disabled" '
 * );</code>
 * @param array $addElements additional input elements as array. Array key = data base field
 * e.g.<code>array(
 *     'id' => ' <input ...> '
 * );</code>
 * @return HTML
 */
function tabprint($arrTabFieldData,$arrMysqlTableStructure,$addStructure=array() ,$addElements=array()){
  $content=print_r("\n  <tr>\n    <th>field</th>\n    <th>value</th>\n    <th></th>\n  </tr>\n",true);
  $i=0;
  foreach ($arrTabFieldData[0] as $key => $value) {
  # fetch all field properties
    $add=get_object_vars($arrMysqlTableStructure[$i]);
  # check for empty fields
    $structure = $add["Null"]=="NO"  && !ereg('disabled',$addStructure[$key]) ? ' onblur="emptyCheck(this);" ' : '';
    $structure.= $addStructure[$key];
    $hidden=ereg('disabled',$addStructure[$key]) ? "\n      <input name=\"".$key."\" value=\"". $value."\" type=\"hidden\">":"";
    $add["Null"] = $add["Null"]=="NO" && !ereg('disabled',$addStructure[$key]) ? '*' : '';
    $structure.= $add["Null"]=="*" ? " class='necessary' " : (ereg('class *=',$addStructure[$key])?"" :" class='defaultinput' ");
  # fields (left)
    $field="<strong><span id='tab".$key."' >".$key.$add["Null"]."</span></strong><br><em>".$add["Comment"]."</em>";
    $field="<td style='border-top:dashed 1px;'><!-- col 1 -->\n      ".$field."\n    </td>";
  # @annotation input values
    # convert all characters
    $value=htmlentities($value,ENT_QUOTES,'UTF-8');
  # @annotation tabprint: check database field types
    $FieldTypes = preg_replace("/^([a-z]+).*/","\\1",$add['Type']);
    switch ($FieldTypes) {
      case 'text':
        $value = "<textarea rows=10 name=\"".$key."\" ".$structure.">".$value."</textarea>";
        break;
      case 'enum':
        $search=array(
          "/enum\((.+)\)/",
          "/'($value)'(,|)/U",
          "/'(.+)'(,|)/U",
        );
        $replace=array(
          "\\1",
          "\n  <option value=\"\\1\" selected=\"true\">\\1</option>",
          "\n  <option value=\"\\1\" >\\1</option>",
        );
        $option = preg_replace($search,$replace,$add['Type']);
        $value = "<select name=\"".$key."\" ".$structure.">\n".$option."\n</select>";
        break;
      case 'varchar':
        $structure.=preg_replace('/varchar\((.+)\)/'," maxlength=\\1 ",$add['Type']);
        $value = "<input name=\"".$key."\" type=\"text\" id=\"".$key."\" value=\"".$value."\" ".$structure."  >";
        break;
      default:
        $value = "<input name=\"".$key."\" type=\"text\" id=\"".$key."\" value=\"".$value."\" ".$structure." >";
    }// end switch
    $value.=$hidden;
    $value.=empty($addElements)?"":$addElements[$key];
    $value="<td style='border-top:dashed 1px;'><!-- col 2 -->\n      ".$value."\n    </td>\n    ";
    $value.= $i==0 ? "\n    <td><!-- col 3 -->\n      <button  onmouseover=\"return overlib('save',WIDTH, 80, OFFSETY, 20, CENTER );\" onmouseout=\"return nd();\"  type='SUBMIT' name='update' value='1'><img src='../structure/img/b_save.png' width='16' height='16'></button>\n    </td>" : "\n    <td><!-- col 3 -->\n    </td>";
  # @annotation tabprint: put content together
    $content.=print_r("\n  <tr><!-- ".($i+1)."-->\n    ".$field ."\n    ". $value."\n  </tr>\n",true);
    $i++;// add 1
  }// end foreach
  return $content;
}//end tabprint

/**
  * Replace difficult code characters
  *
  * For instance with javascript overlib it collapse to have in abstracts or
  * notes a single quote
  */
function replaceHTMLcode($string){
  # replace quotes: " ' -> ” and ’
  $string=str_replace('\"','”',$string);
  $string=str_replace("\'",'’',$string);
  $string=str_replace('&quot;','”',$string);
  $string=str_replace('"','”',$string);
  $string=str_replace("&#039;",'’',$string);
  $string=str_replace("'",'’',$string);
  return $string;
}//end funktionsname

/**
  * replace linebreaks to <br>
  * @param string $string
  * @param string $replace what to replace
  */
function lb2string($string,$replace='<br>')
{
  return preg_replace('/[\n\r]{2}/',$replace,$string);
}

/**
 * Read files in a directory
 *
 * @param string $strDir searched directory (relative)
 * @param string $strFileToSearch which file to search
 * @param boolean $isSimple if true then with $LastModified
 * @return array $arrFiles array of files
 */

function FilesRead($strDir,$strFileToSearch='',$isSimple=false)
{// read only current directory
 $arrFiles = array();
 $It =  opendir($strDir);
 if (! $It)
  die('Cannot list files for ' . $strDir);
 while ($Filename = readdir($It))
  {
    if ($Filename == '.' || $Filename == '..')
    continue;
    $LastModified = filemtime($strDir . $Filename);
      //$arrFiles[] = array($strDir .$Filename, $LastModified);
    if(!empty($strFileToSearch)){
      if (preg_match("/".$strFileToSearch."/i", $Filename)) {
        // get only desired filenames in the array
        $arrFiles[]=$isSimple == false ? array($Filename, $LastModified) : $Filename;
      }
    } else {
      // get any filenames in the array
        $arrFiles[]=$isSimple == false ? array($Filename, $LastModified) : $Filename;
    }
  }
  // return the array
  return $arrFiles;
}// end function FilesRead


/**
 * Put things into an array
 *
 * Helper function used in regular expression search to creates an array usable for
 * instance by {@link $_SESSION['references']}
 *
 * @param string $content
 * @param boolean $print do print?
 * @param array &$arrayToPut put in which array
 * @return array|content
 */


  function putToArray($content,&$arrayToPut,$print=false){
    // &$arrayToPut takes the original variable name when use putToArray($text,$_SESSIO['name'])
    $arrayToPut[]=$content;
    if($print) return $content; // out put possible
  }

/**
 * Print text as <samp><pre>...</pre></samp>
 * @param string $strContent content to print
 * @param boolean $doReturn return string or print directly
 */

  function print_pre($strContent, $doReturn=false){
    $content = "\n<pre class='info'>\n";
    $content.= print_r($strContent, true);
    $content.= "\n</pre>\n";
    if($doReturn)
      return $content;
    else
      print_r($content);
  }

/**
 * Sort comma lists by patternSplit or arrays to strings
 * @param mixed $words string with commas or array
 * @param bool $isUnique should sorting be unique?
 * @param string $patternSplit the pattern to split the string
 * @param string $patternJoin the pattern to join the string again
 * @return string|false
 */
function user_sortwords($words, $isUnique = TRUE, $patternSplit=" *, +", $patternJoin=", "){
  // $words = (array) $words; // switch to array
  // $words = (integer) $words; // switch to integer
  // $words = (float) $words; // switch to float
  // $words = (bool) $words; // switch to true/false
  // $words = (string) $words; // switch to string array() becomes only 'Array'!!
  if(empty($words))
    return false;// return + stop here!!
  if(is_string($words) || is_array($words)){
# is string -> array
  if(is_string($words))
    $words = split(" *, +", $words);
# make unique
  if($isUnique)
    $words = array_unique($words);
# sorting
  natcasesort($words);
    $words = implode(", ", $words);
# give back
    return $words;
  } else
    return false; // something other than array or string + stop here!!
  // further statements
  // __METHOD__ class::methodname
  // __FUNCTION__ methodname
}// end function user_sortwords()

?>
